//-----------------------------------------------------------------------------
// Premake - gnu.c
//
// GNU tool target.
//
// Copyright (C) 2002-2005 by Jason Perkins
// Source code licensed under the GPL, see LICENSE.txt for details.
//
// $Id: gnu.c,v 1.42 2005/08/18 14:35:32 jason379 Exp $
//-----------------------------------------------------------------------------

#include <stdio.h>
#include <string.h>
#include "project.h"
#include "util.h"

#define MAKEFILE_EXT   ".make"
#define OBJECTS_EXT    ".o"

extern const char** commandLineArgs;
extern const char* rootProjectFile;
extern const char* cc;
extern const char* dotnet;
extern int verbose;

static int writeRootMakefile();
static int writeCppPackage(Package* package);
static int writeCsPackage(Package* package);

//-----------------------------------------------------------------------------

int makeGnuScripts()
{
	int i;

	puts("Generating GNU makefiles:");
	
	if (!writeRootMakefile())
		return 0;

	for (i = 0; i < project->numPackages; ++i)
	{
		int result;
		Package* package = project->package[i];
		const char* language = package->language;

		printf("...%s\n", package->name);

		if (strcmp(language, "c#") == 0)
			result = writeCsPackage(package);
		else if (strcmp(language, "c++") == 0 || strcmp(language, "c") == 0)
			result = writeCppPackage(package);
		else
		{
			printf("** Error: unrecognized language '%s'\n", language);
			return 0;
		}

		if (!result) return 0;
	}

	return 1;
}

//-----------------------------------------------------------------------------

static int packageOwnsPath(Package* package)
{
	int i;
	if (strcmp(project->path, package->path) == 0)
		return 0;
	for (i = 0; i < project->numPackages; ++i)
	{
		if (project->package[i] != package &&
			  strcmp(project->package[i]->path, package->path) == 0)
			return 0;
	}
	return 1;
}

static const char* checkDeps(const char* ref, void* data)
{
	int i;
	for (i = 0; i < project->numPackages; ++i)
	{
		if (strcmp(project->package[i]->name, ref) == 0)
			return project->package[i]->name;
	}
	return NULL;
}

static int writeRootMakefile()
{
	int i;
	FILE* file;

	file = openFile(project->path, "Makefile", "");
	if (file == NULL)
	{
		puts("Error: unable to open Makefile\n");
		return 0;
	}

	// Print the header, showing the possible build configurations

	fprintf(file, "# Makefile autogenerated by premake\n");
	fprintf(file, "# Options:\n");
	fprintf(file, "#   CONFIG=[");
	for (i = 0; i < project->package[0]->numConfigs; ++i)
	{
		if (i > 0) fprintf(file, "|");
		fprintf(file, project->package[0]->config[i]->name);
	}
	
	fprintf(file, "]\n\n");

	// Set a default build config if none specified
	
	fprintf(file, "ifndef CONFIG\n");
	fprintf(file, "  CONFIG=%s\n", project->package[0]->config[0]->name);
	fprintf(file, "endif\n\n");
	fprintf(file, "export CONFIG\n\n");
	
	// List all of the available make targets

	fprintf(file, ".PHONY: all clean");
	for (i = 0; i < project->numPackages; ++i)
		fprintf(file, " %s", project->package[i]->name);
	fprintf(file, "\n\n");

	// Make rules

	fprintf(file, "all:");
	for (i = 0; i < project->numPackages; ++i)
		fprintf(file, " %s", project->package[i]->name);
	fprintf(file, "\n\n");

	// Target to regenerate the makefiles when the premake scripts change

	fprintf(file, "Makefile: %s%s", reversePath(project->path, ".", UNIX, 1), rootProjectFile);
	for (i = 0; i < project->numPackages; ++i)
	{
		if (project->package[i]->script != NULL)
			fprintf(file, " %s%s", reversePath(project->path, ".", UNIX, 1), project->package[i]->script);
	}
	fprintf(file, "\n");
	fprintf(file, "\t@echo ==== Regenerating Makefiles ====\n");
	fprintf(file, "\t@premake");
	for (i = 1; args[i] != NULL; ++i)
		fprintf(file, " %s", args[i]);
	fprintf(file, "\n\n");

	/* Individual package targets */

	for (i = 0; i < project->numPackages; ++i)
	{
		Package* package = project->package[i];
		
		fprintf(file, "%s:", package->name);
			writeList(file, package->config[0]->links, " ", "", "", checkDeps, NULL);
			fprintf(file, "\n");

		fprintf(file, "\t@echo ==== Building %s ====\n", package->name);
		fprintf(file, "\t@$(MAKE) ");
		fprintf(file, "--no-print-directory -C %s", reversePath(project->path, package->path, UNIX, 1));
		if (!packageOwnsPath(package))
			fprintf(file, " -f %s%s\n", package->name, MAKEFILE_EXT);
		fprintf(file, "\n");
	}
	
	fprintf(file, "clean:\n");
	for (i = 0; i < project->numPackages; ++i)
	{
		Package* package = project->package[i];

		fprintf(file, "\t@$(MAKE) ");
		fprintf(file, "--no-print-directory -C %s", reversePath(project->path, package->path, UNIX, 1));
		if (!packageOwnsPath(package))
			fprintf(file, " -f %s%s", package->name, MAKEFILE_EXT);
		fprintf(file, " clean\n");
	}
	
	fclose(file);
	return 1;
}

//-----------------------------------------------------------------------------

static const char* checkCppSources(const char* file, void* data)
{
	if (isCppFile(file))
		return replaceExtension(getFilename(file,0), OBJECTS_EXT);
	return NULL;
}

static const char* checkCppFlags(const char* flag, void* data)
{
	char* cc = (char*)data;

	if (strcmp(cc, "LD") == 0)
	{
		if (strcmp(flag, "strip") == 0)
			return "-s";
		else
			return NULL;
	}
	else if (strcmp(cc, "C++") == 0)
	{
		if (strcmp(flag, "no-rtti") == 0)
			return "--no-rtti";
		else if (strcmp(flag, "no-exceptions") == 0)
			return "--no-exceptions";
		else
			return NULL;
	}
	else
	{
		if (strcmp(flag, "with-symbols") == 0)
			return "-g";
		else if (strcmp(flag, "no-frame-pointer") == 0)
			return "-fomit-frame-pointer";
		else if (strcmp(flag, "optimize") == 0)
			return "-O2";
		else if (strcmp(flag, "optimize-speed") == 0)
			return "-O3";
		else if (strcmp(flag, "optimize-size") == 0)
			return "-Os";
		else if (strcmp(flag, "warnings") == 0)
			return "-Wall";
		else if (strcmp(flag, "extra-warnings") == 0)
			return "-W";
		else if (strcmp(flag, "fatal-warnings") == 0)
			return "-Werror";
		else
			return NULL;
	}
}

static void writeSourcePaths(FILE* file, const char* path, int stage)
{
	if (stage == WST_OPENGROUP && strlen(path) > 0)
		fprintf(file, " %s", path);
}

static const char* checkLink(const char* path, void* data)
{
	Package* package = getPackage(path);
	if (package == NULL) return path;

	if (strcmp(package->language, "c++") == 0 || strcmp(package->language, "c") == 0)
		return package->config[*((int*)data)]->target;
	
	return NULL;
}

static int writeCppPackage(Package* package)
{
	const char* prefix = (verbose>1) ? "" : "@";
	const char* comprefix = (verbose) ? "" : "@";

	char buffer[4096];
	FILE* file;
	int i;

	const char* name = package->name;
	const char* path = package->path;
	const char* kind = package->kind;

	/* Open package makefile and write the header */

	if (packageOwnsPath(package))
		file = openFile(package->path, "Makefile", "");
	else
		file = openFile(package->path, package->name, MAKEFILE_EXT);
	if (file == NULL) return 0;

	fprintf(file, "# Sub-makefile autogenerated by premake\n\n");

	/* Set a default configuration */

	fprintf(file, "ifndef CONFIG\n");
	fprintf(file, "  CONFIG=%s\n", package->config[0]->name);
	fprintf(file, "endif\n\n");

	fprintf(file, "# Forward declare all of my variables\n");
	fprintf(file, "OUTDIR =\n");
	fprintf(file, "OBJDIR =\n");
	fprintf(file, "BINDIR =\n");
	fprintf(file, "LIBDIR =\n");
	fprintf(file, "CFLAGS =\n");
	fprintf(file, "CPPFLAGS =\n");
	fprintf(file, "LDFLAGS =\n");
	
	/* Process the build configurations */
	for (i = 0; i < package->numConfigs; ++i)
	{
		ProjectConfig* prjCfg = project->config[i];
		Config* config = package->config[i];

		/* Set up the flags specific to the type of binary being built */
		const char* cflags = "";
		const char* ldflags = "";
		const char* location = "$(BINDIR)";
		if (strcmp(kind, "exe") == 0 || strcmp(kind, "winexe") == 0)
		{
			/* defaults are fine */
		}
		else if (strcmp(kind, "dll") == 0)
		{
			if (strcmp(osIdent, "windows") != 0)
				cflags = " -fPIC";
			if (cc == NULL || strcmp(cc, "gcc") == 0)
				ldflags = " -shared";
		}
		else if (strcmp(kind, "lib") == 0)
		{
			location = "$(LIBDIR)";
		}
		else
		{
			printf("** Error: unknown package type '%s'\n", kind);
			return 0;
		}

		fprintf(file, "ifeq ($(CONFIG),%s)\n", config->name);

		strcpy(buffer, reversePath(path, prjCfg->bindir, UNIX, 1));
		buffer[strlen(buffer)-1] = '\0';
		fprintf(file, "  BINDIR = %s\n", buffer);
  
		strcpy(buffer, reversePath(path, prjCfg->libdir, UNIX, 1));
		buffer[strlen(buffer)-1] = '\0';
		fprintf(file, "  LIBDIR = %s\n", buffer);
		
		fprintf(file, "  OUTDIR = %s\n", location);
		fprintf(file, "  OBJDIR = %s\n\n", package->objdir);

		fprintf(file, "  CPPFLAGS =");
		if (cc == NULL || strcmp(cc, "dmc") != 0)
			fprintf(file, " -MD");
		writeList(file, config->defines, " -D \"", "\"", "", NULL, NULL);
		writeList(file, config->includePaths, " -I ",   "", "", NULL, NULL);
		fprintf(file, "\n");

		fprintf(file, "  CFLAGS += $(CPPFLAGS)");
		writeList(file, config->buildOptions, " ", "", "", NULL, "C");
		writeList(file, config->buildFlags, " ", "", "", checkCppFlags, "C");
		fprintf(file, "\n");

		fprintf(file, "  CXXFLAGS = $(CFLAGS)");
		writeList(file, config->buildFlags, " ", "", "", checkCppFlags, "C++");
		fprintf(file, "\n");

		fprintf(file, "  LDFLAGS += %s", ldflags);
		if (strcmp(osIdent, "macosx") == 0 && inArray(config->buildFlags, "dylib"))
			fprintf(file, " -dynamiclib -flat_namespace");
		writeList(file, config->linkOptions, " ", "", "", NULL, NULL);
		writeList(file, config->libPaths, " -L ", "", "", NULL, NULL);
		writeList(file, config->links, " -l", "", "", checkLink, &i);
		writeList(file, config->linkFlags, " ", "", "", checkCppFlags, "LD");
		fprintf(file, "\n");

		fprintf(file, "  TARGET = ");
		if (strcmp(kind, "winexe") == 0 || strcmp(kind, "exe") == 0)
		{
			if (strcmp(kind, "winexe") == 0 && strcmp(osIdent, "macosx") == 0)
			{
				fprintf(file, "%s.app/Contents/MacOS/%s\n", config->target, config->target);
				fprintf(file, "  MACAPP = %s\n", config->target);
			}
			else
			{
				fprintf(file, "%s\n", config->target);
			}
		}
		else
		{
			fprintf(file, "%s/", getDirectory(config->target));
			if (strcmp(kind, "dll") == 0)
			{
				const char* filename = getFilename(config->target, 0);
				if (strcmp(osIdent, "windows") == 0)
				{
					fprintf(file, "%s.%s\n", filename, config->extension ? config->extension : "dll");
				}
				else if (strcmp(osIdent, "macosx") == 0 && inArray(config->buildFlags, "dylib"))
				{
					fprintf(file, "%s.%s\n", filename, config->extension ? config->extension : "dylib");
				}
				else
				{
					fprintf(file, "lib%s.%s\n", filename, config->extension ? config->extension : "so");
				}
			}
			else
			{
				fprintf(file, "lib%s.%s\n", getFilename(config->target, 0), config->extension ? config->extension : "a");
			}
		}

		fprintf(file, "endif\n");
	}

	fprintf(file, "\n");

	/* These rules auto-create dependency lists (see "Advanced Auto-Dependency
	 * Generation at http://www.paulandlesley.org/gmake/autodep.html) and place
	 * the *.P and *.o files in $(OBJ_DIR)
	 *
	 * Note all of these targets do the same thing. Is there a way to reuse the
	 * command block? Didn't see anything in the make manual, though it would
	 * seem this is something people would want to do. */

	for (i = 0; CPP_EXT[i]; ++i)
	{
		fprintf(file, "$(OBJDIR)/%%.o : %%%s\n", CPP_EXT[i]);
		fprintf(file, "\t-%sif [ ! -d $(OBJDIR) ]; then mkdir -p $(OBJDIR); fi\n", prefix);
		if (!verbose)
			fprintf(file, "\t@echo $(notdir $<)\n");

		if (cc != NULL && strcmp(cc, "dmc") == 0)
		{
			/* FIXME: How to handle assembly files with DMC? */
			if (strcmp(CPP_EXT[i], ".c") == 0)
				fprintf(file, "\t%sdmc $(CFLAGS) -o $@ -c $<\n", comprefix);
			else if (strcmp(CPP_EXT[i], ".s") != 0)
				fprintf(file, "\t%sdmc -cpp -Ae -Ar -mn -D_WINDOWS $(CXXFLAGS) -o $@ -c $<\n", comprefix);
		}
		else
		{
			if (strcmp(CPP_EXT[i], ".s") == 0)
				fprintf(file, "\t%s$(CC) -x assembler-with-cpp $(CPPFLAGS) -o $@ -c $<\n", comprefix);
			else if (strcmp(CPP_EXT[i], ".c") == 0)
				fprintf(file, "\t%s$(CC) $(CFLAGS) -o $@ -c $<\n", comprefix);
			else
				fprintf(file, "\t%s$(CXX) $(CXXFLAGS) -o $@ -c $<\n", comprefix);
		}

		fprintf(file, "\t-%sif [ -f $*.d ]; then mv $*.d $(OBJDIR)/$*.d; fi\n", prefix);
		fprintf(file, "\t%scp $(OBJDIR)/$*.d $(OBJDIR)/$*.P; \\\n", prefix);
		fprintf(file, "\t sed -e 's/#.*//' -e 's/^[^:]*: *//' -e 's/ *\\\\$$//' \\\n");
		fprintf(file, "\t -e '/^$$/ d' -e 's/$$/ :/' < $(OBJDIR)/$*.d >> $(OBJDIR)/$*.P; \\\n");
		fprintf(file, "\trm -f $(OBJDIR)/$*.d\n\n");
	}


	/* Write out the list of object file targets for all C/C++ sources */
	fprintf(file, "OBJECTS = \\\n");
	writeList(file, package->files, "\t$(OBJDIR)/", " \\\n", "", checkCppSources, NULL);
	fprintf(file, "\n");

	/* Write out the list of source directories */
	fprintf(file, "VPATH =");
	walkSourceList(file, package, "", writeSourcePaths);
	fprintf(file, "\n\n");

	fprintf(file, ".PHONY: clean\n");
	fprintf(file, "\n");

	if (strcmp(osIdent, "macosx") == 0 && strcmp(kind, "winexe") == 0)
	{
		fprintf(file, "all: $(BINDIR)/$(MACAPP).app/Contents/PkgInfo $(BINDIR)/$(MACAPP).app/Contents/Info.plist $(BINDIR)/$(TARGET)\n\n");
	}

	fprintf(file, "$(OUTDIR)/$(TARGET): $(OBJECTS)\n");
	if(!verbose)
		fprintf(file, "\t@echo Linking %s\n", package->name);
	fprintf(file, "\t-%sif [ ! -d $(BINDIR) ]; then mkdir -p $(BINDIR); fi\n", prefix);
	fprintf(file, "\t-%sif [ ! -d $(LIBDIR) ]; then mkdir -p $(LIBDIR); fi\n", prefix);
	if (strcmp(osIdent, "macosx") == 0 && strcmp(kind, "winexe") == 0)
		fprintf(file, "\t-%sif [ ! -d $(BINDIR)/$(MACAPP).app/Contents/MacOS ]; then mkdir -p $(BINDIR)/$(MACAPP).app/Contents/MacOS; fi\n", prefix);

	if (strcmp(kind, "lib") == 0)
	{
		fprintf(file, "\t%sar -cr $@ $^\n", comprefix);
		fprintf(file, "\t%sranlib $@\n", comprefix);
	}
	else
	{
		fprintf(file, "\t%s$(CXX) -o $@ $(filter-out %%.so, $^) $(LDFLAGS)\n", comprefix);
	}
	fprintf(file, "\n");

	if (strcmp(osIdent, "macosx") == 0 && strcmp(kind, "winexe") == 0)
	{
		fprintf(file, "$(BINDIR)/$(MACAPP).app/Contents/PkgInfo:\n");
		fprintf(file, "\n");

		fprintf(file, "$(BINDIR)/$(MACAPP).app/Contents/Info.plist:\n");
		fprintf(file, "\n");
	}

	fprintf(file, "clean:\n");
	fprintf(file, "\t@echo Cleaning %s\n", package->name);
	if (strcmp(osIdent, "macosx") == 0 && strcmp(kind, "winexe") == 0)
	{
		fprintf(file, "\t-%srm -rf $(OUTDIR)/$(MACAPP).app $(OBJDIR)/*\n", comprefix);
	}
	else
	{
		fprintf(file, "\t-%srm -rf $(OUTDIR)/$(TARGET) $(OBJDIR)/*\n", comprefix);
	}
	fprintf(file, "\n");

	fprintf(file, "-include $(OBJECTS:%%.o=%%.P)\n\n");

	fclose(file);
	return 1;
}

//-----------------------------------------------------------------------------

static const char* checkReferences(const char* file, void* data)
{
	Package* package = getPackage(file);
	if (package == NULL) return file;
	if (strcmp(package->language, "c#") != 0) return NULL;
	return package->config[*((int*)data)]->target;
}

typedef struct BuildActionData
{
	Package* package;
	const char* buildAction;
	const char* fileExtension;
} BuildActionData;

int csMatchFile(const char* fileName, BuildActionData* data)
{
	FileConfig* config = getFileConfig(data->package, fileName);
	return(
		(	0 != config->buildAction	
			&& 0 == strcmp(config->buildAction, data->buildAction) ) ||
		(	0 != data->fileExtension 
			&& 0 == strcmp(getExtension(fileName), data->fileExtension) ) );
}

static const char* checkCsBuildAction(const char* file, void* data)
{
	BuildActionData* bad = (BuildActionData*)data;
	if(csMatchFile(file, bad))
	{
		return file;
	}
	return NULL;
}

static void writeCsCopyActions(FILE* file, const char** list, BuildActionData* data)
{
	while(*list)
	{
		const char* value = *list;
		if(csMatchFile(value, data))
		{
			fprintf(file, "$(BINDIR)/%s: %s\n\t-@cp -fR %s $(BINDIR)/%s",
				value, value, value, value);
		}
		++list;
	}
}

static int writeCsPackage(Package* package)
{
	BuildActionData bad;
	const char* extension;
	const char* csc;
	FILE* file;
	int i;

	// Figure out what I'm building

	const char* name = package->name;
	const char* path = package->path;
	const char* kind = package->kind;

	if (strcmp(kind, "dll") == 0 || strcmp(kind, "aspnet") == 0)
	{
		extension = ".dll";
		kind = "library";
	}
	else if (strcmp(kind, "winexe") == 0 || strcmp(kind, "exe") == 0)
	{
		extension = ".exe";
	}
	else
	{
		printf("** Error: unknown package type '%s'\n", kind);
		return 0;
	}

	/* Figure out what I'm building it with */

	if (dotnet == NULL)
	{
		if (strcmp(osIdent, "windows") == 0)
			csc = "csc /nologo";
		else
			csc = "mcs";
	}
	else if (strcmp(dotnet, "ms") == 0)
		csc = "csc";
	else if (strcmp(dotnet, "pnet") == 0)
		csc = "cscc";
	else if (strcmp(dotnet, "mono") == 0)
		csc = "mcs";
	else
	{
		printf("** Error: unknown .NET runtime '%s'\n", dotnet);
		return 0;
	}

	/* Open package makefile and write the header */

	if (packageOwnsPath(package))
		file = openFile(package->path, "Makefile", "");
	else
		file = openFile(package->path, package->name, MAKEFILE_EXT);
	if (file == NULL) return 0;

	fprintf(file, "# Sub-makefile autogenerated by premake\n\n");

	/* Set a default configuration */
	fprintf(file, "ifndef CONFIG\n");
	fprintf(file, "  CONFIG=%s\n", package->config[0]->name);
	fprintf(file, "endif\n\n");

	fprintf(file, "BINDIR =\n");
	fprintf(file, "FLAGS =\n");
	fprintf(file, "TARGET =\n\n");

	// Process the build configurations

	for (i = 0; i < package->numConfigs; ++i)
	{
		ProjectConfig* prjCfg = project->config[i];
		Config* config = package->config[i];

		int symbols = !inArray(config->buildFlags,"no-symbols");
		int optimize = inArray(config->buildFlags,"optimize")||inArray(config->buildFlags,"optimize-size")||inArray(config->buildFlags,"optimize-speed");
		int unsafe = inArray(config->buildFlags,"unsafe");

		fprintf(file, "ifeq ($(CONFIG),%s)\n", config->name);

		fprintf(file, "BINDIR = %s\n\n", reversePath(path, prjCfg->bindir, UNIX, 1));
	
		fprintf(file, "  FLAGS +=");
		if (symbols) fprintf(file, " /debug");
		if (unsafe) fprintf(file, " /unsafe");
		writeList(file, config->defines, " /d:", "", "", NULL, NULL);
		writeList(file, config->libPaths, " /lib:", "", "", NULL, NULL);
		writeList(file, config->links, " /r:", ".dll", "", checkReferences, &i);
		fprintf(file, "\n");

		fprintf(file, "  TARGET = %s%s\n", config->target, extension);
		fprintf(file, "endif\n");
	}
	
	// Create the list of files
	
	fprintf(file, "\nSOURCES = \\\n");
	bad.package = package;
	bad.fileExtension = ".cs";
	bad.buildAction = "Code";
	writeList(file, package->files, "\t", " \\\n", "", checkCsBuildAction, &bad);
	fprintf(file, "\n");
	bad.fileExtension = ".resx";
	bad.buildAction = "EmbeddedResource";
	fprintf(file, "\nEMBEDDEDFILES = \\\n");
	writeList(file, package->files, "\t", " \\\n", "", checkCsBuildAction, &bad);
	fprintf(file, "\n");
	fprintf(file, "\nEMBEDDEDCOMMAND = \\\n");
	writeList(file, package->files, "\t/resource:", " \\\n", "", checkCsBuildAction, &bad);
	fprintf(file, "\n");
	bad.buildAction = "LinkedResource";
	bad.fileExtension = NULL;
	fprintf(file, "\nLINKEDCOMMAND = \\\n");
	writeList(file, package->files, "\t/linkresource:", " \\\n", "", checkCsBuildAction, &bad);
	fprintf(file, "\nCOMPILECOMMAND = $(SOURCES) $(EMBEDDEDCOMMAND) $(LINKEDCOMMAND)\n");


	// Build targets 
	fprintf(file, "all: \\\n");
	writeList(file, package->files, "\t$(BINDIR)/", " \\\n", "", checkCsBuildAction, &bad);
	bad.buildAction = "Content";
	writeList(file, package->files, "\t$(BINDIR)/", " \\\n", "", checkCsBuildAction, &bad);
	fprintf(file, "\t$(BINDIR)/$(TARGET)\n");
	fprintf(file, "\n# Distributed files\n");
	// Linked
	bad.buildAction = "LinkedResource";
	writeCsCopyActions(file, package->files, &bad);
	// Content
	bad.buildAction = "Content";
	writeCsCopyActions(file, package->files, &bad);
	
	
	fprintf(file, "\n.PHONY: clean\n\n");

	fprintf(file, "$(BINDIR)/$(TARGET): $(SOURCES) $(RESOURCEFILES)\n");
	fprintf(file, "\t-@if [ ! -d $(BINDIR) ]; then mkdir -p $(BINDIR); fi\n");
	fprintf(file, "\t@%s /out:$@ /t:%s -lib:$(BINDIR) $(FLAGS) $(COMPILECOMMAND)\n\n", csc, kind);

	fprintf(file, "clean:\n");
	fprintf(file, "\t@echo Cleaning %s\n", name);
	fprintf(file, "\t-@rm -f  $(BINDIR)/$(TARGET)\n");
	bad.buildAction = "LinkedResource";
	writeList(file, package->files, "\t-@rm -Rf $(BINDIR)/", " \n", "", checkCsBuildAction, &bad);
	bad.buildAction = "Content";
	writeList(file, package->files, "\t-@rm -Rf $(BINDIR)/", " \n", "", checkCsBuildAction, &bad);

	fclose(file);
	return 1;
}
